//
// Created by marti on 2018-10-15.
//

#ifndef PRGASST5_SAFEINTARRAY_H
#define PRGASST5_SAFEINTARRAY_H

using std::string;

class SafeIntArray {
protected:
    int* theArray;
    int size;

public:
    // Constructors
    SafeIntArray(int size=0);
    SafeIntArray(SafeIntArray& toCopy);

    // Destructor
    ~SafeIntArray();

    // Getter methods
    int getSize();
    int getElement(int index);

    // Setter method
    void setElement(int index, int element);

    // Using reference return value to both get and set
    int& at(int index);
    int& operator[](int index);

    // operators overloaded
    SafeIntArray& operator=(const SafeIntArray& toAssign);
    SafeIntArray operator+(SafeIntArray& toAdd);

    // TODO for PrgAsst5

    /**
     * Converts the array into a string
     * @return A string containing all the elements of the array, separated by commas, surrounded by curly braces
     * For example, if the array contains elements 0 1 2, then calling to_string on that array should return
     * "{0,1,2}"
     */
     string to_string();

     /**
      * Slice similar to the operator on Python
      * @param start The starting index of the slice (inclusive)
      * @param end The ending index of the slice (exclusive)
      * @param inc The increment of the slice, has a default value of 1
      * @return A SafeIntArray containing the elements of the slice
      * @throws runtime_error if the start or end indices are out of bounds
      * For example, if the array conatins the elements 0 1 2 3 4 5 6 7 8 9, then calling slice(2,7,3)
      * should return a SafeIntArray of size 2 containing 2 5
      */
      SafeIntArray slice(int start, int end, int inc=1);

      /**
       * Repetition operator, similar to Python
       * @param rep The number of repetitions to be applied
       * @return A SafeIntArray containing rep repetitions of this one
       */
       SafeIntArray operator*(int rep);

      /**
       * Determines if two SafeIntArrays are equal
       * @param other The SafeIntArray to which this one is compared
       * @return true if the two SafeIntArrays are equal, false otherwise
       */
       bool operator==(SafeIntArray& other);

       /**
        * Finds the index of the first occurrence of an clement
        * @param value The element the method should search for
        * @return an integer indicating the index of the first occurrence of the element. The function should return
        *         -1 if the element is not in the array
        */
        int find(int value);

        /**
         * Returns a new SafeIntArray whose content is in the reverse order
         * @return A SafeIntArray whose content is the same as the SafeIntArray, but with the order reversed.
         * For example, if the array contains the elements 0 1 2 3 4 5, then calling reverse should return a new
         * SafeIntArray whose content is 5 4 3 2 1 0.
         */
         SafeIntArray reverse();
};

#endif //PRGASST5_SAFEINTARRAY_H
